
//*****************************************************************************}
//                                                                             }
//       Password Manager plugin                                               }
//       kpmIAutofill implementation using JS-ctypes                           }
//                                                                             }
//        1997-2012 Kaspersky Lab ZAO                                         }
//                                                                             }
//*****************************************************************************}

// import additional utils
Components.utils.import('resource://gre/modules/XPCOMUtils.jsm');
Components.utils.import('resource://gre/modules/ctypes.jsm');

// debug modes
const kpmDebug_Errors = true;

const kpmLineBreak = '\r\n';

// kpmLog -----------------------------------------------------------------------

const kpmLog =
{
  consoleService: Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService),
  
  logMessage: function(text)
  {
    try 
    {
      this.consoleService.logStringMessage('Kaspersky PURE: ' + text);
    }
    catch (ErrorMessage)
    {
      // keep silence
    }
  },
  
  logError: function(text)
  {
    if (kpmDebug_Errors)
    try 
    {
      Components.utils.reportError('Kaspersky PURE: ' + text);
    }
    catch (ErrorMessage)
    {
      // keep silence
    }
  }
};

// kpmAutofill ------------------------------------------------------------------

// obtain ABI
var WinABI;
var CallBackABI;
if (ctypes.size_t.size == 8)
{
  WinABI = ctypes.default_abi;
  CallBackABI = ctypes.default_abi;
}
else
{
  WinABI = ctypes.winapi_abi;
  CallBackABI = ctypes.stdcall_abi;
}

// declare types
const VOID = ctypes.void_t;
const DWORD = ctypes.uint32_t;
const PDWORD = ctypes.uint32_t.ptr;  
const PWideChar = ctypes.jschar.ptr;
const PPWideChar = ctypes.jschar.ptr.ptr;
const Pointer = ctypes.voidptr_t;
const BOOL = ctypes.bool;
const PBOOL = ctypes.bool.ptr;
const Integer = ctypes.int32_t;
const PInteger = ctypes.int32_t.ptr;

function kpmGetNPAPIPluginFileName()
{
  var Result = '';
  try
  {
    // open library
    var advapi32dll = ctypes.open('advapi32.dll');
    
    // declare types
    const HKEY = ctypes.uint32_t;
    const PHKEY = ctypes.uint32_t.ptr;
    
    // declare constants
    const HKEY_CURRENT_USER = 2147483649; // $80000001
    const KEY_READ = 131097;
    const ERROR_SUCCESS = 0;
    const REG_SZ = 1;
    const MAX_PATH = 260;
    
    // declare functions
    var RegOpenKeyEx = advapi32dll.declare('RegOpenKeyExW', WinABI, 
      ctypes.int32_t,  // Result
      HKEY,            // hKey
      PWideChar,       // lpSubKey
      DWORD,           // ulOptions
      DWORD,           // samDesired 
      PHKEY            // phkResult
    );
    var RegCloseKey = advapi32dll.declare('RegCloseKey', WinABI, 
      ctypes.int32_t,  // Result
      HKEY             // hKey
    );
    var RegQueryValueEx = advapi32dll.declare('RegQueryValueExW', WinABI, 
      ctypes.int32_t,  // Result
      HKEY,            // hKey
      PWideChar,       // lpValueName
      Pointer,         // lpReserved
      PDWORD,          // lpType
      PWideChar,       // lpData, WARNING: used for this case PWideChar instead of PByte
      PDWORD           // lpcbData
    );

    // try to open key
    var Key = HKEY(0);
    var RegResult = RegOpenKeyEx(HKEY_CURRENT_USER,
      'Software\\MozillaPlugins\\@kaspersky.com/Kaspersky PURE', 0, KEY_READ, Key.address());
    if (RegResult == ERROR_SUCCESS)
    {
      Key = Key.address().contents; // retrieve the value of key from it's address
      // try to read value
      var PathSize = DWORD(2 * MAX_PATH); // size in bytes!
      var DataType = DWORD(REG_SZ);
      var lpPath = new new ctypes.ArrayType(ctypes.jschar, MAX_PATH);
      if (RegQueryValueEx(Key, 'Path', null, DataType.address(), lpPath, PathSize.address()) == ERROR_SUCCESS)
        Result = lpPath.readString();
      // close key
      RegCloseKey(Key);
    }
    
    // close library
    advapi32dll.close();
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmGetNPAPIPluginFileName() Error: ' + ErrorMessage);
    Result = '';
  }
  return Result;
}

//class constructor
function kpmAutofill()
{
  try
  {
    this.spPluginDllInitialized = false;
    this.spPluginDll = null;

    var FileName = kpmGetNPAPIPluginFileName();
    if (FileName)
    {
      // open library
      this.spPluginDll = ctypes.open(FileName);

      // declare functions
      //   base
      const TspStrDispose = ctypes.FunctionType(CallBackABI,
        VOID,            // Result
        [
        PWideChar        // lpStr
        ]
      );
      const TkpmIsCaptionButtonMenuShowed = ctypes.FunctionType(CallBackABI,
        BOOL            // Result
      );
      const TkpmShowCaptionButtonMenu = ctypes.FunctionType(CallBackABI,
        VOID            // Result
      );      
      const TkpmIsWindowFocused = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PWideChar,      // WindowTitle
        Integer,        // WindowScreenX
        Integer         // WindowScreenY
        ]
      );
      const TspCalcElementScreenRect = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        // in
        PWideChar,      // ActiveTabLocationName
        // inout
        PInteger,       // Left
        PInteger,       // Top
        PInteger,       // Right
        PInteger        // Bottom
        ]
      );
      const TkpmSetUrl = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        PWideChar       // Url
        ]
      );
      const TspHideShowedCaptionButtonBalloon = ctypes.FunctionType(CallBackABI,
        VOID            // Result
      );
      const TkpmAutofillMultiloginAccount = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        PWideChar       // Url
        ]
      );
      const TkpmIsIgnoredPasswordHint = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PWideChar,      // Url
        PWideChar       // Password
        ]
      );
      const TkpmIsMultipageAuthorizationLogin = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PWideChar,      // DocumentUrl
        PWideChar,      // FormUrl
        PWideChar,      // ElementID
        PWideChar       // ElementName
        ]
      );
      const TspStoreMultipageAuthorizationInfo = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        PWideChar,      // Url
        PWideChar       // Login
        ]
      );
      const TkpmAllowVirtualKeyboardAutoShow = ctypes.FunctionType(CallBackABI,
        BOOL            // Result
      );
      const TkpmShowVirtualKeyboard = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        Integer,        // ControlRectLeft
        Integer,        // ControlRectTop
        Integer,        // ControlRectRight
        Integer         // ControlRectBottom
        ]
      );
      const TspAutoHideVirtualKeyboard = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        Integer,        // ControlRectLeft
        Integer,        // ControlRectTop
        Integer,        // ControlRectRight
        Integer         // ControlRectBottom
        ]
      );
      const TkpmAllowShowPasswordGeneratorBalloon = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PWideChar,      // Url
        PWideChar,      // DOMXml
        PInteger        // AllowUseHistory
        ]
      );
      const TkpmShowPasswordGeneratorBalloon = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        Integer,        // PasswordControlRectLeft
        Integer,        // PasswordControlRectTop
        Integer,        // PasswordControlRectRight
        Integer,        // PasswordControlRectBottom
        BOOL            // AllowUseHistory
        ]
      );
      const TspAutoHidePasswordGeneratorBalloon = ctypes.FunctionType(CallBackABI,
        VOID            // Result
      );
      const TkpmPromptMailPassword = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        // in
        DWORD,          // UrlScheme
        PWideChar,      // DialogTitle
        PWideChar,      // DialogText
        PWideChar,      // DialogTextTemplate
        PWideChar,      // DialogTextTemplate1
        // out
        PPWideChar,     // lpPassword
        PInteger        // UseRealEngine
        ]
      );
      const TkpmPromptLoginAndPassword = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        // in
        PWideChar,      // Url
        PWideChar,      // DialogText
        // inout
        PPWideChar,     // lpLogin
        // out
        PPWideChar,     // lpPassword
        PInteger        // UseRealEngine
        ]
      );
      const TkpmBeforeSubmit = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        // in
        PWideChar,      // Url
        PWideChar,      // LocationName
        PWideChar,      // Login
        PWideChar,      // Password
        PWideChar,      // DOMXml
        PWideChar,      // PasswordControlFormAction
        DWORD,          // PasswordControlFormMethod
        // out
        PInteger,       // Cancel
        PPWideChar      // lpBeforeSubmitManagerXml
        ]
      );
      const TkpmDocumentComplete = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        // in
        PWideChar,      // AuthKey
        PWideChar,      // ActiveTabLocationName
        PWideChar,      // Url
        PWideChar,      // LocationName
        PWideChar,      // DOMXml
        BOOL,           // IsActiveTab
        BOOL,           // PasswordControlExists
        Integer,        // PasswordControlRectLeft
        Integer,        // PasswordControlRectTop
        Integer,        // PasswordControlRectRight
        Integer,        // PasswordControlRectBottom
        PWideChar,      // PasswordControlFormAction
        DWORD,          // PasswordControlFormMethod
        // out
        PInteger,       // IdentityFormIndex
        PDWORD,         // AutofillReason
        PPWideChar,     // AccountID
        PPWideChar,     // AccountName
        PPWideChar,     // Login
        PPWideChar,     // RealLogin
        PPWideChar,     // Password
        PPWideChar,     // AdditionalFieldsXml
        PInteger,       // UrlIsAccountLink
        PInteger,       // AllowSetFocus
        PInteger        // AutoSubmit
        ]
      );
      const TkpmAutofillCallback = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        PWideChar,      // ActiveTabLocationName
        BOOL,           // LoginAutofilled
        BOOL,           // PasswordAutofilled
        BOOL,           // IdentityAutofilling
        PWideChar,      // AccountID
        PWideChar,      // AccountName
        PWideChar       // Login
        ]
      );
      const TkpmIsPasswordManagerUnlocked = ctypes.FunctionType(CallBackABI,
        BOOL            // Result
      );
      //   Listener engine
      const TspHasWebBrowserTabMessage = ctypes.FunctionType(CallBackABI,
        BOOL            // Result
      );
      const TspMessageProcessFailed = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        DWORD           // MessageID
        ]
      );
      const TspMustSetActiveElementSelectedText = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PPWideChar      // lpSelectedText
        ]
      );
      const TspMustGetActiveElementScreenRect = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PDWORD          // MessageID
        ]
      );
      const TkpmGetActiveElementScreenRect = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        DWORD,          // MessageID
        Integer,        // Left
        Integer,        // Top
        Integer,        // Right
        Integer,        // Bottom
        BOOL            // IsTextElement
        ]
      );
      const TspMustGetDOMXml = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PDWORD          // MessageID
        ]
      );
      const TkpmGetDOMXml = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        DWORD,          // MessageID
        PWideChar       // DOMXml
        ]
      );
      const TspMustExecuteUrl = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        // in
        PWideChar,      // ActiveTabLocationName
        // out
        PDWORD,         // MessageID
        PPWideChar,     // lpUrl
        PInteger        // InActiveTab
        ]
      );
      const TspExecuteUrl = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        DWORD,          // MessageID
        BOOL            // Result
        ]
      );
      const TspMustSetIsActiveDocumentComplete = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        PDWORD          // MessageID
        ]
      );
      const TkpmSetIsActiveDocumentComplete = ctypes.FunctionType(CallBackABI,
        VOID,           // Result
        [
        DWORD,          // MessageID
        BOOL            // Result
        ]
      );
      const TspMustManualAutofill = ctypes.FunctionType(CallBackABI,
        BOOL            // Result
      );
      const TkpmManualAutofill = ctypes.FunctionType(CallBackABI,
        BOOL,           // Result
        [
        // in
        PWideChar,      // AuthKey
        // out
        PInteger,       // IdentityAutofilling
        PInteger,       // IdentityFormIndex
        PDWORD,         // AutofillReason
        PPWideChar,     // AccountID
        PPWideChar,     // AccountName
        PPWideChar,     // Login
        PPWideChar,     // RealLogin
        PPWideChar,     // Password
        PPWideChar,     // AdditionalFieldsXml
        PInteger,       // UrlIsAccountLink
        PInteger,       // AllowSetFocus
        PInteger        // AutoSubmit
        ]
      );      
      
      const spPluginFunctionsSignature = 0xDB4109C2;
      
      const TspPluginFunctions = new ctypes.StructType('TspPluginFunctions',
      [ 
        {'Signature': DWORD},
        // base
        {'StrDispose': TspStrDispose.ptr},
        {'IsCaptionButtonMenuShowed': TkpmIsCaptionButtonMenuShowed.ptr},
        {'ShowCaptionButtonMenu': TkpmShowCaptionButtonMenu.ptr},        
        {'IsWindowFocused': TkpmIsWindowFocused.ptr},
        {'CalcElementScreenRect': TspCalcElementScreenRect.ptr},
        {'SetUrl': TkpmSetUrl.ptr},
        {'HideShowedCaptionButtonBalloon': TspHideShowedCaptionButtonBalloon.ptr},
        {'AutofillMultiloginAccount': TkpmAutofillMultiloginAccount.ptr},
        {'IsIgnoredPasswordHint': TkpmIsIgnoredPasswordHint.ptr},
        {'IsMultipageAuthorizationLogin': TkpmIsMultipageAuthorizationLogin.ptr},
        {'StoreMultipageAuthorizationInfo': TspStoreMultipageAuthorizationInfo.ptr},
        {'AllowVirtualKeyboardAutoShow': TkpmAllowVirtualKeyboardAutoShow.ptr},
        {'ShowVirtualKeyboard': TkpmShowVirtualKeyboard.ptr},
        {'AutoHideVirtualKeyboard': TspAutoHideVirtualKeyboard.ptr},
        {'AllowShowPasswordGeneratorBalloon': TkpmAllowShowPasswordGeneratorBalloon.ptr},
        {'ShowPasswordGeneratorBalloon': TkpmShowPasswordGeneratorBalloon.ptr},
        {'AutoHidePasswordGeneratorBalloon': TspAutoHidePasswordGeneratorBalloon.ptr},
        {'PromptMailPassword': TkpmPromptMailPassword.ptr},
        {'PromptLoginAndPassword': TkpmPromptLoginAndPassword.ptr},
        {'BeforeSubmit': TkpmBeforeSubmit.ptr},
        {'DocumentComplete': TkpmDocumentComplete.ptr},
        {'AutofillCallback': TkpmAutofillCallback.ptr},
        {'IsPasswordManagerUnlocked': TkpmIsPasswordManagerUnlocked.ptr},
        // Listener engine
        {'HasWebBrowserTabMessage': TspHasWebBrowserTabMessage.ptr},
        {'MessageProcessFailed': TspMessageProcessFailed.ptr},
        {'MustSetActiveElementSelectedText': TspMustSetActiveElementSelectedText.ptr},
        {'MustGetActiveElementScreenRect': TspMustGetActiveElementScreenRect.ptr},
        {'GetActiveElementScreenRect': TkpmGetActiveElementScreenRect.ptr},
        {'MustGetDOMXml': TspMustGetDOMXml.ptr},
        {'GetDOMXml': TkpmGetDOMXml.ptr},
        {'MustExecuteUrl': TspMustExecuteUrl.ptr},
        {'ExecuteUrl': TspExecuteUrl.ptr},
        {'MustSetIsActiveDocumentComplete': TspMustSetIsActiveDocumentComplete.ptr},
        {'SetIsActiveDocumentComplete': TkpmSetIsActiveDocumentComplete.ptr},
        {'MustManualAutofill': TspMustManualAutofill.ptr},
        {'ManualAutofill': TkpmManualAutofill.ptr}
      ]);
      const PspPluginFunctions = TspPluginFunctions.ptr;
      
      this.spPluginFunctions = new TspPluginFunctions();
      this.spPluginFunctions.Signature = spPluginFunctionsSignature;
      
      var spInitializePlugin = this.spPluginDll.declare('spInitializePlugin', WinABI,
        BOOL,              // Result
        PspPluginFunctions // lpPluginFunctions
      );
      if (spInitializePlugin(this.spPluginFunctions.address()))
      {
        // all functions loaded successfully
        this.spPluginDllInitialized = true;
      }
    }
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmAutofill() Error: ' + ErrorMessage);
  }
};

// class definition
kpmAutofill.prototype =
{
  // this must match whatever is in chrome.manifest!
  classID: Components.ID('{98914FC4-3E30-4864-A554-01FB82804F60}'),

  // dll
  spPluginDllInitialized: false,  
  spPluginDll: null,
  spPluginFunctions: null,

  StrToPWideChar: function(Str)
  {
    if (Str)
      return ctypes.jschar.array()(Str);
    else
      return "";
  },

  PWideCharToStrAndDispose: function(lpStr)
  {
    // lpStr: PWideChar
    var Result = '';
    try
    {  
      if (lpStr && lpStr.address() && lpStr.address().contents)
      {
        Result = lpStr.readString();
        this.spPluginFunctions.StrDispose(lpStr.address().contents);
      }
    }
    catch (ErrorMessage)
    {
      // keep silence
      Result = '';
    }    
    return Result;
  },

  // nsISupports implementation

  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.kpmIAutofill]),

  // kpmIAutofill implementation

  //   base

  // PRBool IsCaptionButtonMenuShowed();
  IsCaptionButtonMenuShowed: function()
  {
    if (!this.spPluginDllInitialized)
      return false;
    return this.spPluginFunctions.IsCaptionButtonMenuShowed();
  },

  // void ShowCaptionButtonMenu();
  ShowCaptionButtonMenu: function()
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.ShowCaptionButtonMenu();
  },

  // PRBool IsWindowFocused(in wstring WindowTitle, in long WindowScreenX, in long WindowScreenY);
  IsWindowFocused: function(WindowTitle, WindowScreenX, WindowScreenY)
  {
    if (!this.spPluginDllInitialized)
      return false;
    return this.spPluginFunctions.IsWindowFocused(this.StrToPWideChar(WindowTitle), Integer(WindowScreenX), Integer(WindowScreenY));
  },

  /*
  PRBool CalcElementScreenRect(
    // in
    in wstring ActiveTabLocationName,
    // inout
    inout long left, inout long top, inout long right, inout long bottom
  );
  */
  CalcElementScreenRect: function(ActiveTabLocationName, left, top, right, bottom)
  {
    if (!this.spPluginDllInitialized)
      return false;
    // 1. obtain params
    //   inout
    var left_ = DWORD(left.value);
    var top_ = DWORD(top.value);
    var right_ = DWORD(right.value);
    var bottom_ = DWORD(boottom.value);
    // 2. call function
    var Result = this.spPluginFunctions.CalcElementScreenRect(this.StrToPWideChar(ActiveTabLocationName),
      left_.address(), top_.address(), right_.address(), bottom_.address());
    // 3. update out params
    if (Result)
    {
      left.value = left_.address().contents;
      top.value = top_.address().contents;
      right.value = right_.address().contents;
      bottom.value = bottom_.address().contents;
    }
    return Result;
  },

  // void SetUrl(in wstring Url);  
  SetUrl: function(Url)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.SetUrl(this.StrToPWideChar(Url));
  },

  // void HideShowedCaptionButtonBalloon();
  HideShowedCaptionButtonBalloon: function()
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.HideShowedCaptionButtonBalloon();
  },

  // void AutofillMultiloginAccount(in wstring Url);
  AutofillMultiloginAccount: function(Url)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.AutofillMultiloginAccount(this.StrToPWideChar(Url));
  },

  // PRBool IsIgnoredPasswordHint(in wstring Url, in wstring Password);
  IsIgnoredPasswordHint: function(Url, Password)
  {
    if (!this.spPluginDllInitialized)
      return;
    return this.spPluginFunctions.IsIgnoredPasswordHint(this.StrToPWideChar(Url), this.StrToPWideChar(Password));
  },

  /*
  PRBool IsMultipageAuthorizationLogin(
    in wstring DocumentUrl, in wstring FormUrl, in wstring ElementID, in wstring ElementName);
  */
  IsMultipageAuthorizationLogin: function(DocumentUrl, FormUrl, ElementID, ElementName)
  {
    if (!this.spPluginDllInitialized)
      return;
    return this.spPluginFunctions.IsMultipageAuthorizationLogin(
      this.StrToPWideChar(DocumentUrl), this.StrToPWideChar(FormUrl),
      this.StrToPWideChar(ElementID), this.StrToPWideChar(ElementName)
    );
  },

  // void StoreMultipageAuthorizationInfo(in wstring Url, in wstring Login);    
  StoreMultipageAuthorizationInfo: function(Url, Login)
  {
    if (!this.spPluginDllInitialized)
      return;
    return this.spPluginFunctions.StoreMultipageAuthorizationInfo(this.StrToPWideChar(Url), this.StrToPWideChar(Login));
  },

  // PRBool AllowVirtualKeyboardAutoShow();
  AllowVirtualKeyboardAutoShow: function()
  {
    if (!this.spPluginDllInitialized)
      return false;
    return this.spPluginFunctions.AllowVirtualKeyboardAutoShow();
  },

  /*
  void ShowVirtualKeyboard(in long ControlRectLeft, in long ControlRectTop,
    in long ControlRectRight, in long ControlRectBottom);
  */
  ShowVirtualKeyboard: function(ControlRectLeft, ControlRectTop, ControlRectRight, ControlRectBottom)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.ShowVirtualKeyboard(Integer(ControlRectLeft), Integer(ControlRectTop), 
      Integer(ControlRectRight), Integer(ControlRectBottom));
  },

  /*
  void AutoHideVirtualKeyboard(in long ControlRectLeft, in long ControlRectTop,
    in long ControlRectRight, in long ControlRectBottom);
  */
  AutoHideVirtualKeyboard: function(ControlRectLeft, ControlRectTop, ControlRectRight, ControlRectBottom)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.AutoHideVirtualKeyboard(Integer(ControlRectLeft), Integer(ControlRectTop), 
      Integer(ControlRectRight), Integer(ControlRectBottom));
  },

  /* PRBool AllowShowPasswordGeneratorBalloon (in wstring Url, in wstring DOMXml, out PRBOOL AllowUseHistory); */
  AllowShowPasswordGeneratorBalloon: function(Url, DOMXml, AllowUseHistory)
  {
    if (!this.spPluginDllInitialized)
      return false;
    // 1. obtain params
    //   inout
    var AllowUseHistory_ = Integer(0);
    // 2. call function
    var Result = this.spPluginFunctions.AllowShowPasswordGeneratorBalloon(
      this.StrToPWideChar(Url), this.StrToPWideChar(DOMXml), 
      AllowUseHistory_.address());
    // 3. update out params
    if (Result)
    {
      AllowUseHistory.value = AllowUseHistory_.address().contents;
    }
    return Result;
  },

  /* void ShowPasswordGeneratorBalloon (
    in long PasswordControlRectLeft, in long PasswordControlRectTop, 
    in long PasswordControlRectRight, in long PasswordControlRectBottom,
    in long AllowUseHistory); */
  ShowPasswordGeneratorBalloon: function(
    PasswordControlRectLeft, PasswordControlRectTop, 
    PasswordControlRectRight, PasswordControlRectBottom, AllowUseHistory)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.ShowPasswordGeneratorBalloon(
      Integer(PasswordControlRectLeft), Integer(PasswordControlRectTop), 
      Integer(PasswordControlRectRight), Integer(PasswordControlRectBottom),
      AllowUseHistory);
  },

  /* void AutoHidePasswordGeneratorBalloon (); */
  AutoHidePasswordGeneratorBalloon: function()
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.AutoHidePasswordGeneratorBalloon();
  },

  /*
  PRBool PromptMailPassword(in long UrlScheme, in wstring DialogTitle,
    in wstring DialogText, in wstring DialogTextTemplate, 
    in wstring DialogTextTemplate1, out wstring Password, out PRBool UseRealEngine);
  */
  PromptMailPassword: function(UrlScheme, DialogTitle, DialogText, DialogTextTemplate, DialogTextTemplate1, 
    Password, UseRealEngine)
  {
    try
    {
      if (!this.spPluginDllInitialized)
      {
        UseRealEngine.value = true;
        return false;
      }
      // 1. obtain params
      var Password_ = PWideChar(0);
      var UseRealEngine_ = Integer(0);
      // 2. call function   
      var Result = this.spPluginFunctions.PromptMailPassword(
        UrlScheme,
        this.StrToPWideChar(DialogTitle),
        this.StrToPWideChar(DialogText),
        this.StrToPWideChar(DialogTextTemplate),
        this.StrToPWideChar(DialogTextTemplate1),
        Password_.address(),
        UseRealEngine_.address()
      );    
      // 3. update out params
      if (Result)
      {
        Password.value = this.PWideCharToStrAndDispose(Password_);
        UseRealEngine.value = UseRealEngine_.address().contents;      
      }
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('PromptMailPassword() Error: ' + ErrorMessage);
      return false;
    }
  },
  
  /*
  PRBool PromptLoginAndPassword(in wstring Url, in wstring DialogText,
    inout wstring Login, out wstring Password, out PRBool UseRealEngine);
  */
  PromptLoginAndPassword: function(Url, DialogText, Login, Password, UseRealEngine)
  {
    try
    {
      if (!this.spPluginDllInitialized)
        return false;
      // 1. obtain params
      var Login_ = PWideChar(0); // !! Login.value
      var Password_ = PWideChar(0);
      var UseRealEngine_ = Integer(0);
      // 2. call function   
      var Result = this.spPluginFunctions.PromptLoginAndPassword(
        this.StrToPWideChar(Url),
        this.StrToPWideChar(DialogText),
        Login_.address(),
        Password_.address(),
        UseRealEngine_.address()
      );
      // 3. update out params
      Login.value = this.PWideCharToStrAndDispose(Login_);
      if (Result)
      {
        Password.value = this.PWideCharToStrAndDispose(Password_);
        UseRealEngine.value = UseRealEngine_.address().contents;      
      }
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('PromptLoginAndPassword() Error: ' + ErrorMessage);
      return false;
    }
  },
 
  /*
  void BeforeSubmit(
    // in
    in wstring Url, in wstring LocationName,
    in wstring Login, in wstring Password, in wstring DOMXml,
    in wstring PasswordControlFormAction, in long PasswordControlFormMethod,
    // out
    out PRBool Cancel, out wstring BeforeSubmitManagerXml);
  */
  BeforeSubmit: function(Url, LocationName, Login, Password, DOMXml, PasswordControlFormAction, PasswordControlFormMethod,
    Cancel, BeforeSubmitManagerXml)
  {
    try
    {
      if (!this.spPluginDllInitialized)
        return;
      // 1. obtain params
      var Cancel_ = Integer(0);
      var BeforeSubmitManagerXml_ = PWideChar(0);
      // 2. call function   
      this.spPluginFunctions.BeforeSubmit(
        this.StrToPWideChar(Url),
        this.StrToPWideChar(LocationName),
        this.StrToPWideChar(Login),
        this.StrToPWideChar(Password),
        this.StrToPWideChar(DOMXml),
        this.StrToPWideChar(PasswordControlFormAction),
        PasswordControlFormMethod,
        Cancel_.address(),
        BeforeSubmitManagerXml_.address()
      );    
      // 3. update out params
      Cancel.value = Cancel_.address().contents;
      BeforeSubmitManagerXml.value = this.PWideCharToStrAndDispose(BeforeSubmitManagerXml_);
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('BeforeSubmit() Error: ' + ErrorMessage);
    }
  },
  
  /*
  PRBool DocumentComplete(
    // in
    in wstring AuthKey,
    in wstring ActiveTabLocationName,
    in wstring Url, in wstring LocationName,
    in wstring DOMXml, in long IsActiveTab,
    in long PasswordControlExists,
    in long PasswordControlRectLeft, in long PasswordControlRectTop,
    in long PasswordControlRectRight, in long PasswordControlRectBottom,
    in wstring PasswordControlFormAction, in long PasswordControlFormMethod,
    // out
    out long IdentityFormIndex, out long AutofillReason,
    out wstring AccountID, out wstring AccountName, out wstring Login, out wstring RealLogin,
    out wstring Password, out wstring AdditionalFieldsXml,
    out PRBool UrlIsAccountLink, out PRBool AllowSetFocus, out PRBool AutoSubmit);
  */
  DocumentComplete: function(
    AuthKey, ActiveTabLocationName, Url, LocationName, DOMXml, IsActiveTab, PasswordControlExists,
    PasswordControlRectLeft, PasswordControlRectTop, PasswordControlRectRight, PasswordControlRectBottom,
    PasswordControlFormAction, PasswordControlFormMethod,
    IdentityFormIndex, AutofillReason, AccountID, AccountName, Login, RealLogin,
    Password, AdditionalFieldsXml, UrlIsAccountLink, AllowSetFocus, AutoSubmit)
  {
    try
    {
      if (!this.spPluginDllInitialized)
        return false;
      // 1. obtain params
      var IdentityFormIndex_ = Integer(0);
      var AutofillReason_ = DWORD(0);
      var AccountID_ = PWideChar(0);
      var AccountName_ = PWideChar(0);
      var Login_ = PWideChar(0);
      var RealLogin_ = PWideChar(0);
      var Password_ = PWideChar(0);
      var AdditionalFieldsXml_ = PWideChar(0);
      var UrlIsAccountLink_ = Integer(0);
      var AllowSetFocus_ = Integer(0);
      var AutoSubmit_ = Integer(0);
      // 2. call function
      var Result = this.spPluginFunctions.DocumentComplete(
        this.StrToPWideChar(AuthKey),
        this.StrToPWideChar(ActiveTabLocationName),
        this.StrToPWideChar(Url),
        this.StrToPWideChar(LocationName),
        this.StrToPWideChar(DOMXml),
        IsActiveTab,
        PasswordControlExists, PasswordControlRectLeft, PasswordControlRectTop,
        PasswordControlRectRight, PasswordControlRectBottom,
        this.StrToPWideChar(PasswordControlFormAction),
        PasswordControlFormMethod,
        IdentityFormIndex_.address(),
        AutofillReason_.address(),
        AccountID_.address(),
        AccountName_.address(),
        Login_.address(),
        RealLogin_.address(),
        Password_.address(),
        AdditionalFieldsXml_.address(),
        UrlIsAccountLink_.address(),
        AllowSetFocus_.address(),
        AutoSubmit_.address()
      );    
      // 3. update out params
      if (Result)
      {
        IdentityFormIndex.value = IdentityFormIndex_.address().contents;
        AutofillReason.value = AutofillReason_.address().contents;
        AccountID.value = this.PWideCharToStrAndDispose(AccountID_);
        AccountName.value = this.PWideCharToStrAndDispose(AccountName_);
        Login.value = this.PWideCharToStrAndDispose(Login_);
        RealLogin.value = this.PWideCharToStrAndDispose(RealLogin_);
        Password.value = this.PWideCharToStrAndDispose(Password_);
        AdditionalFieldsXml.value = this.PWideCharToStrAndDispose(AdditionalFieldsXml_);
        UrlIsAccountLink.value = UrlIsAccountLink_.address().contents;
        AllowSetFocus.value = AllowSetFocus_.address().contents;
        AutoSubmit.value = AutoSubmit_.address().contents;
      }
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('DocumentComplete() Error: ' + ErrorMessage);
      return false;
    }
  },
  
  /*
  void AutofillCallback(in wstring ActiveTabLocationName,
    in long LoginAutofilled, in long PasswordAutofilled, in long IdentityAutofilling,
    in wstring AccountID, in wstring AccountName, in wstring Login);
  */
  AutofillCallback: function(ActiveTabLocationName, LoginAutofilled,
    PasswordAutofilled, IdentityAutofilling, AccountID, AccountName, Login)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.AutofillCallback(
      this.StrToPWideChar(ActiveTabLocationName), 
      BOOL(LoginAutofilled), BOOL(PasswordAutofilled), BOOL(IdentityAutofilling),
      this.StrToPWideChar(AccountID),
      this.StrToPWideChar(AccountName),
      this.StrToPWideChar(Login)
    );
  },

  /* PRBool IsPasswordManagerUnlocked (); */
  IsPasswordManagerUnlocked: function ()
  {
    if (!this.spPluginDllInitialized)
      return false;
    try
    {
      var Result = this.spPluginFunctions.IsPasswordManagerUnlocked();
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('IsPasswordManagerUnlocked() Error: ' + ErrorMessage);
      return false;
    }
  },
  
  //   Listener engine
  
  // void MessageProcessFailed(in long MessageID);
  MessageProcessFailed: function(MessageID)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.MessageProcessFailed(MessageID);
  },
  
  // PRBool MustSetActiveElementSelectedText(out wstring SelectedText);
  MustSetActiveElementSelectedText: function(SelectedText)
  {
    try
    {
      if (!this.spPluginDllInitialized)
        return false;
      // 1. obtain params
      var SelectedText_ = PWideChar(0);
      // 2. call function   
      var Result = this.spPluginFunctions.MustSetActiveElementSelectedText(
        SelectedText_.address()
      );    
      // 3. update out params
      if (Result)
      {
        SelectedText.value = this.PWideCharToStrAndDispose(SelectedText_);
      }
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('MustSetActiveElementSelectedText() Error: ' + ErrorMessage);
      return false;
    }
  },
  
  // PRBool MustGetActiveElementScreenRect(out long MessageID);
  MustGetActiveElementScreenRect: function(MessageID)
  {
    if (!this.spPluginDllInitialized)
      return false;
    // 1. obtain params
    var MessageID_ = DWORD(0);
    // 2. call function
    var Result = this.spPluginFunctions.MustGetActiveElementScreenRect(MessageID_.address());
    // 3. update out params
    MessageID.value = MessageID_.address().contents;
    return Result;
  },
  
  /*
  void GetActiveElementScreenRect(in long MessageID,
    in long left, in long top, in long right, in long bottom, in PRBool IsTextElement);
  */
  GetActiveElementScreenRect: function(MessageID, left, top, right, bottom, IsTextElement)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.GetActiveElementScreenRect(MessageID, Integer(left), Integer(top), 
      Integer(right), Integer(bottom), BOOL(IsTextElement));
  },
  
  // PRBool MustGetDOMXml(out long MessageID);
  MustGetDOMXml: function(MessageID)
  {
    if (!this.spPluginDllInitialized)
      return false;
    // 1. obtain params
    var MessageID_ = DWORD(0);
    // 2. call function
    var Result = this.spPluginFunctions.MustGetDOMXml(MessageID_.address());
    // 3. update out params
    MessageID.value = MessageID_.address().contents;
    return Result;
  },
  
  // void GetDOMXml(in long MessageID, in wstring DOMXml);
  GetDOMXml: function(MessageID, DOMXml)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.GetDOMXml(MessageID, this.StrToPWideChar(DOMXml));
  },
  
  /*
  PRBool MustExecuteUrl(
    // in
    in wstring ActiveTabLocationName,
    // out
    out long MessageID, out wstring Url, out PRBool InActiveTab);
  */
  MustExecuteUrl: function(ActiveTabLocationName, MessageID, Url, InActiveTab)
  {        
    try
    {
      if (!this.spPluginDllInitialized)
        return false;	 
      // 1. obtain params
      var MessageID_ = DWORD(0);
      var Url_ = PWideChar(0);
      var InActiveTab_ = Integer(0);
      // 2. call function  
      var Result = this.spPluginFunctions.MustExecuteUrl(
        this.StrToPWideChar(ActiveTabLocationName),
        MessageID_.address(),
        Url_.address(),
        InActiveTab_.address()
      );    
      // 3. update out params
      if (Result)
      {
        MessageID.value = MessageID_.address().contents;
        Url.value = this.PWideCharToStrAndDispose(Url_);
        InActiveTab.value = InActiveTab_.address().contents; 
      }
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('MustExecuteUrl() Error: ' + ErrorMessage);  
      return false;
    }           
  },
  
  // void ExecuteUrl(in long MessageID, in PRBool Result);  
  ExecuteUrl: function(MessageID, Result)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.ExecuteUrl(MessageID, Result);
  },
  
  // PRBool MustSetIsActiveDocumentComplete(out long MessageID);
  MustSetIsActiveDocumentComplete: function(MessageID)
  {
    if (!this.spPluginDllInitialized)
      return false;
    // 1. obtain params
    var MessageID_ = DWORD(0);
    // 2. call function
    var Result = this.spPluginFunctions.MustSetIsActiveDocumentComplete(MessageID_.address());
    // 3. update out params
    MessageID.value = MessageID_.address().contents;
    return Result;
  },
  
  // void SetIsActiveDocumentComplete(in long MessageID, in PRBool Result);
  SetIsActiveDocumentComplete: function(MessageID, Result)
  {
    if (!this.spPluginDllInitialized)
      return;
    this.spPluginFunctions.SetIsActiveDocumentComplete(MessageID, Result);
  },
  
  // PRBool MustManualAutofill();
  MustManualAutofill: function()
  {
    if (!this.spPluginDllInitialized)
      return false;
    return this.spPluginFunctions.MustManualAutofill();
  },
  
  /*
  PRBool ManualAutofill(
    // in
    in wstring AuthKey,
    // out
    out PRBool IdentityAutofilling, out long IdentityFormIndex, out long AutofillReason,
    out wstring AccountID, out wstring AccountName, out wstring Login, out wstring RealLogin,
    out wstring Password, out wstring AdditionalFieldsXml,
    out PRBool UrlIsAccountLink, out PRBool AllowSetFocus, out PRBool AutoSubmit);
  */
  ManualAutofill: function(
    AuthKey,
    IdentityAutofilling, IdentityFormIndex, AutofillReason,
    AccountID, AccountName, Login, RealLogin,
    Password, AdditionalFieldsXml,
    UrlIsAccountLink, AllowSetFocus, AutoSubmit)
  {
    try
    {
      if (!this.spPluginDllInitialized)
        return false;
      // 1. obtain params
      var IdentityAutofilling_ = Integer(0);
      var IdentityFormIndex_ = Integer(0);
      var AutofillReason_ = DWORD(0);
      var AccountID_ = PWideChar(0);
      var AccountName_ = PWideChar(0);
      var Login_ = PWideChar(0);
      var RealLogin_ = PWideChar(0);
      var Password_ = PWideChar(0);
      var AdditionalFieldsXml_ = PWideChar(0);
      var UrlIsAccountLink_ = Integer(0);
      var AllowSetFocus_ = Integer(0);
      var AutoSubmit_ = Integer(0);
      // 2. call function   
      var Result = this.spPluginFunctions.ManualAutofill(
        this.StrToPWideChar(AuthKey),
        IdentityAutofilling_.address(),
        IdentityFormIndex_.address(),
        AutofillReason_.address(),
        AccountID_.address(),
        AccountName_.address(),
        Login_.address(),
        RealLogin_.address(),
        Password_.address(),
        AdditionalFieldsXml_.address(),
        UrlIsAccountLink_.address(),
        AllowSetFocus_.address(),
        AutoSubmit_.address()
      );    
      // 3. update out params
      if (Result)
      {
        IdentityAutofilling.value = IdentityAutofilling_.address().contents;
        IdentityFormIndex.value = IdentityFormIndex_.address().contents;
        AutofillReason.value = AutofillReason_.address().contents;
        AccountID.value = this.PWideCharToStrAndDispose(AccountID_);
        AccountName.value = this.PWideCharToStrAndDispose(AccountName_);
        Login.value = this.PWideCharToStrAndDispose(Login_);
        RealLogin.value = this.PWideCharToStrAndDispose(RealLogin_);
        Password.value = this.PWideCharToStrAndDispose(Password_);
        AdditionalFieldsXml.value = this.PWideCharToStrAndDispose(AdditionalFieldsXml_);
        UrlIsAccountLink.value = UrlIsAccountLink_.address().contents;
        AllowSetFocus.value = AllowSetFocus_.address().contents;
        AutoSubmit.value = AutoSubmit_.address().contents;
      }
      return Result;
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('ManualAutofill() Error: ' + ErrorMessage);
      return false;
    }
  }
};

// generate factory ------------------------------------------------------------

// XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
if (XPCOMUtils.generateNSGetFactory)
  var NSGetFactory = XPCOMUtils.generateNSGetFactory([kpmAutofill]);