
//*****************************************************************************}
//                                                                             }
//       Password Manager plugin                                               }
//       nsIPromptService implementation                                       }
//                                                                             }
//        1997-2012 Kaspersky Lab ZAO                                         }
//                                                                             }
//*****************************************************************************}

// debug modes
const kpmDebug_Errors = true;
const kpmDebug_StringBundleErrors = false;
const kpmDebug_StringBundle = false;
const kpmDebug_QueryInterface = false;
const kpmDebug_prompt = false;
const kpmDebug_promptUsernameAndPassword = false;
const kpmDebug_promptUsernameAndPasswordUrl = false;
const kpmDebug_promptPassword = false;
const kpmDebug_promptAuth = false;
const kpmDebug_NSGetModule = false;

const kpmLineBreak = '\r\n';

function kpmWidePrepareStr_ps(Text)
{
  var Result;
  try
  {
    if (Text && Text.value != undefined)
      Result = new String(Text.value); // String object
    else
      Result = new String(Text);       // String
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmWidePrepareStr_ps() Error: ' + ErrorMessage);
    Result = new String(Text); // String
  }
  return Result;
}

function kpmWidePrepareLowerCaseStr_ps(Text)
{
  var Result = kpmWidePrepareStr_ps(Text);
  Result = Result.toLowerCase();
  return Result;  
}

function kpmWideIsEmpty_ps(Text)
{
  try
  {
    // kpmLog.logMessage('kpmWideIsEmpty_ps() Text: <' + Text + '>');
    if (Text == '' || Text == null || Text == undefined)
      return true;
    var Str = kpmWidePrepareStr_ps(Text);
    // kpmLog.logMessage('kpmWideIsEmpty_ps() Str: <' + Str + '>');
    if (Str == '' || Str == undefined)
      return true;
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmWideIsEmpty_ps() Error: ' + ErrorMessage);
  }
  return false;  
}

function kpmWideSameText_ps(Text1, Text2)
{
  try
  {
    var Str1 = kpmWidePrepareLowerCaseStr_ps(Text1);
    var Str2 = kpmWidePrepareLowerCaseStr_ps(Text2);
    return Str1 == Str2;
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmWideSameText_ps() Error: ' + ErrorMessage);
    return false;
  }
}

function kpmWideSearchText_ps(ASearchingText, AText)
{
  try
  {
    var SearchingText = kpmWidePrepareLowerCaseStr_ps(ASearchingText);
    var Text = kpmWidePrepareLowerCaseStr_ps(AText);
    if (Text.indexOf(SearchingText) >= 0)
      return true;
    else
      return false;
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmWideSearchText_ps() Error: ' + ErrorMessage);
    return false;
  }
}

function kpmLogStringBundle(StringBundle)
{
  if (!StringBundle)
    return;
  try
  {
    var i = 0;
    var items = StringBundle.getSimpleEnumeration();
    var item;
    while (items.hasMoreElements())
    {
      item = items.getNext();
      item = item.QueryInterface(Components.interfaces.nsIPropertyElement);
      kpmLog.logMessage('StringBundle[' + i + '] Key: <' + item.key +'>, Value: <' + item.value + '>');
      i++;
    }
  }
  catch (ErrorMessage)
  {
    if (kpmDebug_StringBundleErrors)
      kpmLog.logError('kpmLogStringBundle() Error: ' + ErrorMessage);
  }
}

function kpmStringBundle_GetStringFromIDorName(StringBundle, ID, Name)
{
  var Result = '';
  if (StringBundle)
  {
    // 0. try to get from ID
    try
    {
      if (kpmDebug_StringBundle)
        kpmLog.logMessage('StringBundle.GetStringFromID(' + ID + ')');
      Result = StringBundle.GetStringFromID(ID);
      if (kpmDebug_StringBundle)
        kpmLog.logMessage('Result: <' + Result + '>');
      return Result;
    }
    catch (ErrorMessage)
    {
      Result = '';
      if (kpmDebug_StringBundleErrors)
        kpmLog.logError('kpmStringBundle_GetStringFromIDorName() Error: ' + ErrorMessage + kpmLineBreak + 'ID: <' + ID + '>');
    }
    // 1. try to get from Name
    if (!kpmWideIsEmpty_ps(Name))
    try
    {
      if (kpmDebug_StringBundle)
        kpmLog.logMessage('StringBundle.GetStringFromName(' + Name + ')');
      Result = StringBundle.GetStringFromName(Name);
      if (kpmDebug_StringBundle)
        kpmLog.logMessage('Result: <' + Result + '>');
      return Result;
    }
    catch (ErrorMessage)
    {
      Result = '';
      if (kpmDebug_StringBundleErrors)
        kpmLog.logError('kpmStringBundle_GetStringFromIDorName() Error: ' + ErrorMessage + kpmLineBreak + 'Name: <' + Name + '>');
    }
  }
  return Result;
}

function kpmGetProxyServerUrl(aSourceUrl, aProxyUrl)
{
  if (aProxyUrl)
  try
  {
    var PrefService = Components.classes['@mozilla.org/preferences;1'].createInstance(Components.interfaces.nsIPref);
    var PrefBranch = PrefService.getBranch('network.proxy.');
    if (PrefBranch)
    {
      var type = PrefBranch.getIntPref('type');
      if (type == 1)
      {
        // obtain protocol
        var protocol = 'http';
        if (kpmWideSearchText_ps('https://', aSourceUrl))
          protocol = 'ssl';
        else if (kpmWideSearchText_ps('ftp://', aSourceUrl))
          protocol = 'ftp';
        else if (kpmWideSearchText_ps('gopher://', aSourceUrl))
          protocol = 'gopher';
        // read host & port
        var host = PrefBranch.getCharPref(protocol);
        if (!kpmWideIsEmpty_ps(host))
        {
          var port = PrefBranch.getIntPref(protocol + '_port');
          if (port)
          {
            aProxyUrl.value = 'moz-proxy://' + host + ':' + port;
            return true;
          }
        }
      }
    }
  }
  catch(ErrorMessage)
  {
    // keep silence
  }
  return false;
}

// kpmLog -----------------------------------------------------------------------

const kpmLog =
{
  consoleService: Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService),
  
  logMessage: function(text)
  {
    try 
    {
      this.consoleService.logStringMessage('Kaspersky PURE: ' + text);
    }
    catch (ErrorMessage)
    {
      // keep silence
    }
  },
  
  logError: function(text)
  {
    if (kpmDebug_Errors)
    try 
    {
      Components.utils.reportError('Kaspersky PURE: ' + text);
    }
    catch (ErrorMessage)
    {
      // keep silence
    }
  }
};

// kpmAutofill ------------------------------------------------------------------

const KPMAUTOFILL_CID = '{98914FC4-3E30-4864-A554-01FB82804F60}';

// kpmPromptService -------------------------------------------------------------

//class constructor
function kpmPromptService()
{
  var RealPromptService;
  // Firefox <= 3
  RealPromptService = Components.classesByID['{a2112d6a-0e28-421f-b46a-25c0b308cbd0}'];
  if (!RealPromptService)
  {
    // Firefox >= 4
    RealPromptService = Components.classesByID['{7ad1b327-6dfa-46ec-9234-f2a620ea7e00}'];
  }
  if (RealPromptService)
  {
    this.RealPromptService = RealPromptService.getService(Components.interfaces.nsIPromptService);
    if ('nsIPromptService2' in Components.interfaces)
      this.RealPromptService2 = RealPromptService.getService(Components.interfaces.nsIPromptService2);
    else
      this.RealPromptService2 = null;
  }
  else
  {
    this.RealPromptService = null;
    this.RealPromptService2 = null;
  }
  this.UseRealEngine = true;
};
 
// class definition
kpmPromptService.prototype =
{
  // this must match whatever is in chrome.manifest!
  classID: Components.ID('{C59D9F86-286F-4DBE-8C2D-27F6617859D5}'),

  RealPromptService: null,
  
  RealPromptService2: null,

  // kpmAutofill
  
  kpmAutofill: null,
  
  kpmInitAutofill: function()
  {
    // check if kpmAutofill already initialized
    if (this.kpmAutofill)
      return true;
    // try to initialize kpmAutofill
    try 
    {
      this.kpmAutofill = Components.classesByID[KPMAUTOFILL_CID].getService(Components.interfaces.kpmIAutofill);
      if (this.kpmAutofill != null)
        return true;
     else
        return false;
    }
    catch (ErrorMessage)
    {
      this.kpmAutofill = null;
      return false;
    }
  },
  
  kpmPromptMailPassword: function(aDialogTitle, aText, aPassword)
  {
    try
    {
      if (kpmDebug_promptPassword)
        kpmLog.logMessage('kpmPromptService.kpmPromptMailPassword() aDialogTitle: <' + aDialogTitle + '>, aText: <' + aText + '>');
      var UrlScheme = 0; // unknown
      var DialogTitleTemplate = '';
      var DialogTextTemplate = '';
      var DialogTextTemplate1 = '';
      var StringBundleService = Components.classes['@mozilla.org/intl/stringbundle;1']
        .getService(Components.interfaces.nsIStringBundleService);
      var StringBundle = null;
      // POP3
      if (!UrlScheme)
      {
        StringBundle = StringBundleService.createBundle('chrome://messenger/locale/localMsgs.properties');
        // DEBUG: kpmLogStringBundle(StringBundle);
        if (StringBundle)
        {
          // Mozilla Thunderbird
          DialogTitleTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 4020, 'pop3EnterPasswordPromptTitle');
          if (!kpmWideIsEmpty_ps(DialogTitleTemplate) &&
              kpmWideSameText_ps(DialogTitleTemplate, aDialogTitle))
          {
            UrlScheme = 1;
            DialogTextTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 4017, 'pop3EnterPasswordPrompt');
            DialogTextTemplate1 = kpmStringBundle_GetStringFromIDorName(StringBundle, 4018, 'pop3PreviouslyEnteredPasswordIsInvalidPrompt');
          }
        }
      }
      // SMTP
      if (!UrlScheme)
      {
        StringBundle = StringBundleService.createBundle('chrome://messenger/locale/messengercompose/composeMsgs.properties');
        if (StringBundle)
        {
          // DEBUG: kpmLogStringBundle(StringBundle);
          // Mozilla Thunderbird
          DialogTitleTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 12542, 'smtpEnterPasswordPromptTitle');
          // Qualcomm Eudora
          if (kpmWideIsEmpty_ps(DialogTitleTemplate))
            DialogTitleTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 12585, '');
          if (!kpmWideIsEmpty_ps(DialogTitleTemplate) &&
              kpmWideSameText_ps(DialogTitleTemplate, aDialogTitle))
          {
            UrlScheme = 2;
            DialogTextTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 12544, 'smtpEnterPasswordPromptWithUsername');
          }
        }
      }
      // IMAP
      if (!UrlScheme)
      {
        StringBundle = StringBundleService.createBundle('chrome://messenger/locale/imapMsgs.properties');
        if (StringBundle)
        {
          // DEBUG: kpmLogStringBundle(StringBundle);
          // Mozilla Thunderbird
          DialogTitleTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 5051, '');
          // Qualcomm Eudora
          // ??
          if (!kpmWideIsEmpty_ps(DialogTitleTemplate) &&
              kpmWideSameText_ps(DialogTitleTemplate, aDialogTitle))
          {
            UrlScheme = 3;
            DialogTextTemplate = kpmStringBundle_GetStringFromIDorName(StringBundle, 5047, '');
          }
        }
      }
      if (UrlScheme)
      {
        if (kpmDebug_promptPassword)
          kpmLog.logMessage('kpmPromptService.kpmPromptMailPassword() DialogTitleTemplate: <' + DialogTitleTemplate + '>, ' + 
            'DialogTextTemplate: <' + DialogTextTemplate + '>, ' +
            'DialogTextTemplate1: <' + DialogTextTemplate1 + '>');
        // check if kpmAutofill available
        if (this.kpmInitAutofill())
        {
          try
          {  
            var objUseRealEngine = {value: true};
            var Result = this.kpmAutofill.PromptMailPassword(UrlScheme, aDialogTitle, aText, 
              DialogTextTemplate, DialogTextTemplate1, aPassword, objUseRealEngine);
            this.UseRealEngine = objUseRealEngine.value;
            return Result;
          }
          catch (ErrorMessage)
          {
            kpmLog.logError('kpmPromptService.kpmPromptMailPassword() Error call kpmAutofill.PromptMailPassword(): ' + ErrorMessage);
          }
        }
      }
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('kpmPromptService.kpmPromptMailPassword() Error: ' + ErrorMessage);
    }
    this.UseRealEngine = true;
    return false;
  },
  
  kpmPromptUsernameAndPassword: function(aParent, aUrl, aText, aUsername, aPassword)
  {
    try
    {
      if (kpmDebug_promptUsernameAndPassword)
        kpmLog.logMessage('kpmPromptService.kpmPromptUsernameAndPassword() aText: <' + aText + '> aUrl: <' + aUrl + '>');
      // check if Url & kpmAutofill available
      if (!kpmWideIsEmpty_ps(aUrl) && this.kpmInitAutofill())
      {
        try
        {  
          var objUseRealEngine = {value: true};
          var Result = this.kpmAutofill.PromptLoginAndPassword(aUrl, aText, aUsername, aPassword, objUseRealEngine);
          this.UseRealEngine = objUseRealEngine.value;
          return Result;
        } 
        catch(ErrorMessage)
        {
          kpmLog.logError('kpmPromptService.kpmPromptUsernameAndPassword() Error call kpmAutofill.PromptLoginAndPassword(): ' + ErrorMessage);
        }
      }
    } 
    catch(ErrorMessage)
    {
      kpmLog.logError('kpmPromptService.kpmPromptUsernameAndPassword() Error: ' + ErrorMessage);
    }
    this.UseRealEngine = true;
    return false;
  },
  
  // nsISupports implementation
  
  QueryInterface: function(aIID)
  {
    if (kpmDebug_QueryInterface)
      kpmLog.logMessage('kpmPromptService.QueryInterface(' + aIID + ')');
    // is device manager active?
    var WindowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
      .getService(Components.interfaces.nsIWindowWatcher);
    if (WindowWatcher.activeWindow &&
        kpmWideSearchText_ps('chrome://pippki/content/', WindowWatcher.activeWindow.location)
       )
    {
      // device manager active - return RealPromptService.QueryInterface(aIID)
      if (kpmDebug_QueryInterface)
        kpmLog.logMessage('  kpmPromptService.QueryInterface(' + aIID + ') device manager active');
      return this.RealPromptService.QueryInterface(aIID);
    } 
    // if aIID is nsISupports || nsIPromptService || nsIPromptService2 than return this,
    // otherwise - call original nsIPromptService.QueryInterface()
    if (aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsIPromptService) ||
        aIID.equals(Components.interfaces.nsIPromptService2)
       )
    {
      if (kpmDebug_QueryInterface)
      {
        if (aIID.equals(Components.interfaces.nsISupports))
          kpmLog.logMessage('  kpmPromptService.QueryInterface(nsISupports) - this');
        if (aIID.equals(Components.interfaces.nsIPromptService))
          kpmLog.logMessage('  kpmPromptService.QueryInterface(nsIPromptService) - this');
        if (aIID.equals(Components.interfaces.nsIPromptService2))
          kpmLog.logMessage('  kpmPromptService.QueryInterface(nsIPromptService2) - this');
      }
      return this;
    }
    // unknown interface - return RealPromptService.QueryInterface(aIID)
    if (kpmDebug_QueryInterface)
      kpmLog.logMessage('  kpmPromptService.QueryInterface(' + aIID + ') unknown interface - this.RealPromptService.QueryInterface(aIID)');
    return this.RealPromptService.QueryInterface(aIID);
  },

  // nsIPromptService implementation

  alert: function(aParent, aDialogTitle, aText)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.alert(aDialogTitle="' + aDialogTitle + '", aText="' + aText + '")');
    this.RealPromptService.alert(aParent, aDialogTitle, aText);
  },

  alertCheck: function(aParent, aDialogTitle, aText, aCheckMsg, aCheckState)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.alertCheck(aDialogTitle="' + aDialogTitle + '", aText="' + aText + '", aCheckMsg="' + aCheckMsg + '")');
    this.RealPromptService.alertCheck(aParent, aDialogTitle, aText, aCheckMsg, aCheckState);
  },

  confirm: function(aParent, aDialogTitle, aText)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.confirm(aDialogTitle="' + aDialogTitle + '", aText="' + aText + '")');
    return this.RealPromptService.confirm(aParent, aDialogTitle, aText);
  },

  confirmCheck: function(aParent, aDialogTitle, aText, aCheckMsg, aCheckState)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.confirmCheck(aDialogTitle="' + aDialogTitle + '", aText="' + aText + '", aCheckMsg="' + aCheckMsg + '")');
    return this.RealPromptService.confirmCheck(aParent, aDialogTitle, aText, aCheckMsg, aCheckState);
  },

  confirmEx: function(aParent, aDialogTitle, aText, aButtonFlags,
    aButton0Title, aButton1Title, aButton2Title, aCheckMsg, aCheckState)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.confirmEx(aDialogTitle="' + aDialogTitle + '", aText="' + aText + 
        '", aButton0Title="' + aButton0Title + '"' +
        '", aButton1Title="' + aButton1Title + '"' +
        '", aButton2Title="' + aButton2Title + '"' +
        '", aCheckMsg="' + aCheckMsg + '")'
      );
    return this.RealPromptService.confirmEx(aParent, aDialogTitle, aText, aButtonFlags, 
      aButton0Title, aButton1Title, aButton2Title, aCheckMsg, aCheckState);
  },

  prompt: function(aParent, aDialogTitle, aText, aValue,  aCheckMsg, aCheckState)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.prompt(aDialogTitle="' + aDialogTitle + '", aText="' + aText + '", aCheckMsg="' + aCheckMsg + '")');
    return this.RealPromptService.prompt(aParent, aDialogTitle, aText, aValue, aCheckMsg, aCheckState);
  },

  select: function(aParent, aDialogTitle, aText, aCount, aSelectList, aOutSelection)
  {
    if (kpmDebug_prompt)
      kpmLog.logMessage('kpmPromptService.select(aDialogTitle="' + aDialogTitle + '", aText="' + aText + '")');
    return this.RealPromptService.select(aParent, aDialogTitle, aText, aCount, aSelectList, aOutSelection);
  },

  promptUsernameAndPassword: function(aParent, aDialogTitle, aText, aUsername, aPassword, aCheckMsg, aCheckState)
  {
    if (kpmDebug_promptUsernameAndPassword)
      kpmLog.logMessage('kpmPromptService.promptUsernameAndPassword(), ' +
        'aUsername: "' + aUsername.value + '"'
      );
    // get Url, specified by kpmWebProgressListener for aParent window
    var Url = '';
    if (aParent != undefined && aParent != null)
    {
      try
      {  
        Url = aParent.kpmUrl;
        if (kpmWideIsEmpty_ps(Url))
        {
          if (kpmDebug_promptUsernameAndPasswordUrl)
            kpmLog.logMessage('kpmPromptService.kpmPromptUsernameAndPassword() aParent.kpmUrl undefined, try to get from document');
          Url = aParent.document.kpmUrl;
        }
        if (kpmWideIsEmpty_ps(Url))
        {
          if (kpmDebug_promptUsernameAndPasswordUrl)
            kpmLog.logMessage('kpmPromptService.kpmPromptUsernameAndPassword() aParent.document.kpmUrl undefined, try to get from aParent.name');
          Url = aParent.name;
        }        
        if (kpmDebug_promptUsernameAndPasswordUrl)
          kpmLog.logMessage('kpmPromptService.kpmPromptUsernameAndPassword() Url: ' + Url + 
            ', aParent.kpmUrl: ' + aParent.kpmUrl +
            ', aParent.document.kpmUrl: ' + aParent.document.kpmUrl +
            ', aParent.name: ' + aParent.name
          );
      }
      catch(ErrorMessage)
      {
        Url = '';
        kpmLog.logError('kpmPromptService.kpmPromptUsernameAndPassword() Error get Url from aParent: ' + ErrorMessage);
      }
    }
    if (this.kpmPromptUsernameAndPassword(aParent, Url, aText, aUsername, aPassword))    
    {
      try
      {
        aCheckState.value = false;
      }
      catch (ErrorMessage)
      {
        // keep silence
      }
      return true;
    } else
    if (this.UseRealEngine)
    {
      return this.RealPromptService.promptUsernameAndPassword(aParent, aDialogTitle, aText, aUsername, aPassword, aCheckMsg, aCheckState);
    } else
    {
      return false;
    }
  },

  promptPassword: function(aParent, aDialogTitle, aText, aPassword, aCheckMsg, aCheckState)
  {
    if (kpmDebug_promptPassword)
      kpmLog.logMessage('kpmPromptService.promptPassword() aDialogTitle="' + aDialogTitle + '", aText="' + aText + '", aCheckMsg="' + aCheckMsg + '"');
    if (this.kpmPromptMailPassword(aDialogTitle, aText, aPassword))
    {
      try
      {
        aCheckState.value = false;
      }
      catch (ErrorMessage)
      {
        // keep silence
      }
      return true;
    } else
    if (this.UseRealEngine)
    {
      return this.RealPromptService.promptPassword(aParent, aDialogTitle, aText, aPassword, aCheckMsg, aCheckState);
    } else
    {
      return false;
    }
  },

  // nsIPromptService2 implementation
  
  promptAuth: function(aParent, aChannel, level, authInfo, checkboxLabel, checkValue)
  {
    if (kpmDebug_promptAuth)
      kpmLog.logMessage('kpmPromptService: nsIPromptService2.promptAuth(), ' + 
        'aChannel.URI.spec: "' + aChannel.URI.spec + '", ' + 
        'authInfo.domain: "' + authInfo.domain + '", ' + 
        'authInfo.authenticationScheme: "' + authInfo.authenticationScheme + '", ' +
        'authInfo.flags: "' + authInfo.flags + '", ' +
        'authInfo.username: "' + authInfo.username + '", ' + 
        'authInfo.realm: "' + authInfo.realm + '"'        
      );
    var aUsername = {};
    var aPassword = {};
    aUsername.value = authInfo.username;
    aPassword.value = authInfo.password;
    var aUrl = aChannel.URI.spec;
    if (authInfo.flags && this.RealPromptService2.AUTH_PROXY == this.RealPromptService2.AUTH_PROXY)
    {
      // extract proxy server from settings
      var aProxyUrl = {};
      if (kpmGetProxyServerUrl(aChannel.URI.spec, aProxyUrl))
        aUrl = aProxyUrl.value;
    }
    if (this.kpmPromptUsernameAndPassword(aParent, aUrl, '"' + authInfo.realm + '"', aUsername, aPassword))
    { 
      // was domain specified?
      var s = new String(aUsername.value);
      if (s.indexOf('\\') != -1)
      {
        // try to split domain & user name
        var aStrings = s.split('\\');
        if (aStrings.length == 2)
        {
          authInfo.domain = aStrings[0];
          aUsername.value = aStrings[1];
        }
      }
      authInfo.username = aUsername.value;
      authInfo.password = aPassword.value;
      try
      {
        checkValue.value = false;
      }
      catch (ErrorMessage)
      {
        // keep silence
      }
      return true;
    } else
    if (this.UseRealEngine)
    {
      return this.RealPromptService2.promptAuth(aParent, aChannel, level, authInfo, checkboxLabel, checkValue);
    } else
    {
      return false;
    }
  },
    
  asyncPromptAuth: function(aParent, aChannel, aCallback, aContext, level, authInfo, checkboxLabel, checkValue)
  {
    if (kpmDebug_promptAuth)
      kpmLog.logMessage('kpmPromptService: nsIPromptService2.asyncPromptAuth()');
    return this.RealPromptService2.asyncPromptAuth(aParent, aChannel, aCallback, aContext, level, authInfo, checkboxLabel, checkValue);
  }
};

// kpmPromptServiceFactory class factory ----------------------------------------

var kpmPromptServiceFactory =
{
  // nsIFactory implementation
  
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new kpmPromptService()).QueryInterface(aIID);
  },
  
  lockFactory: function (lock)
  {
    // empty
  }
};

// kpmPromptServiceModule module definition (xpcom registration) ----------------
  
var kpmPromptServiceModule =
{
  myClassID: Components.ID('{C59D9F86-286F-4DBE-8C2D-27F6617859D5}'),
  myClassName: 'Kaspersky PURE Prompt Service',
  myContractID: '@mozilla.org/embedcomp/prompt-service;1',
 
  allowReigster: function()
  {
    // deny to register *autofill.xpi if imautofill.xpi already installed!
    if (('kpm' != 'im') &&
        Components.classesByID['{3772011F-C5EC-4E6D-AFCA-50A7DF47AD76}']
       )
    {
      return false;
    }
    return true;
  },
 
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (!this.allowReigster())
      return;
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(this.myClassID, this.myClassName, this.myContractID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    if (!this.allowReigster())
      return;
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(this.myClassID, aLocation);
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aCID.equals(this.myClassID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    return kpmPromptServiceFactory;
  },

  canUnload: function(aCompMgr)
  {
    if (kpmDebug_NSGetModule)
      kpmLog.logMessage('Unloading kpmPromptServiceModule component...');
    return true;
  }
};

// NSGetModule() module initialization -----------------------------------------

function NSGetModule(aCompMgr, aFileSpec)
{
  if (kpmDebug_NSGetModule)
    kpmLog.logMessage('NSGetModule()');
  return kpmPromptServiceModule;
}

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

// XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
if (XPCOMUtils.generateNSGetFactory)
  var NSGetFactory = XPCOMUtils.generateNSGetFactory([kpmPromptService]);