// XpiInstaller
// By Pike (Heavily inspired by code from Henrik Gemal and Stephen Clavering)

const EInstall_202_1 = 'You need to have write permissions to the chrome directory and subfolders:';
const EInstall_202_2 = 'and to the components directory:';
const EInstall_210_1 = 'Installation cancelled by user';
const EInstallUnknown_1 = 'An unknown error occured. Error code: ';
const EInstallUnknown_2 = 'Look at the following URL for a description of the error code:';
const EInstallUnknown_3 = 'http://developer.netscape.com/docs/manuals/xpinstall/err.html';

var XpiInstaller = {

  // XPI Description Info >>
  extFullName: 'Password Manager plugin', // The name displayed to the user (don't include the version)
  extShortName: 'kpmautofill', // The leafname of the JAR file (without the .jar part)
  extVersion: '7.0.3.6',
  extAuthor: 'Kaspersky Lab',
  extComponentNames: ['kpmAutofill.dll', 'kpmAutofill.xpt', 'kpmIPromptService.js', 'kpmIPromptService.xpt'], // e.g. ['MyComponent.js', 'MyComponent.xpt']
  extLocaleNames: null, // e.g. ['en-US', 'en-GB']
  extSkinNames: ['classic'], // e.g. ['classic', 'modern']
  extPostInstallMessage: null, // 'Please, restart your browser to finish the installation.', // Set to null for no post-install message
  // XPI Description Info <<
	
  silentInstall: false,
  chromeFlag: PROFILE_CHROME,
  chromeDir: getFolder('Profile', 'chrome'),
  componentsDir: getFolder('Components'),
  programDir: getFolder('Program'),
	
  install: function()
  {
    // install extension to the Web-browser chrome direcory instead of profile one
    this.chromeFlag = DELAYED_CHROME;
    this.chromeDir = getFolder('chrome');
  
    var jarName = this.extShortName + '.jar';
    var jarPath = getFolder(this.chromeDir, jarName);

    // Init install
    var err = initInstall(this.extFullName + ' ' + this.extVersion, '/' + this.extShortName, this.extVersion);
    resetError();

    // Add JAR file
    addFile(null, 'chrome/' + jarName, this.chromeDir, null);
    err = getLastError();

    // Add components files
    if (this.allowProceedInstall(err)) 
    {
      // addFile(null, 'components/.autoreg', this.programDir, null);
      for (var component in this.extComponentNames)
        addFile(null, 'components/' + this.extComponentNames[component], this.componentsDir, null);
      err = getLastError();
    }

    // Register chrome
    if (this.allowProceedInstall(err)) 
    {
      //   0. Register content
      registerChrome(this.chromeFlag | CONTENT, jarPath, 'content/' + this.extShortName + '/');
      //   1. Register locales
      for (var locale in this.extLocaleNames)
      {
        var regPath = 'locale/' + this.extLocaleNames[locale] + '/' + this.extShortName + '/';
        registerChrome(this.chromeFlag | LOCALE, jarPath, regPath);
      }
      //   2. Register skins
      for (var skin in this.extSkinNames)
      {
        var regPath = 'skin/' + this.extSkinNames[skin] + '/' + this.extShortName + '/';
        registerChrome(this.chromeFlag | SKIN, jarPath, regPath);
      }
      err = getLastError();
    }

    // Perform install
    err = performInstall();
    if (this.allowProceedInstall(err)) 
    {
      if (!this.silentInstall && this.extPostInstallMessage)
        alert(this.extPostInstallMessage);
    } else
    { 
      this.handleError(err);
    }
  },

  allowProceedInstall: function(err)
  {
    if (err == SUCCESS || err == REBOOT_NEEDED)
      return true;
    else
      return false;
  },
	
  handleError: function(err)
  {
    cancelInstall();
    if (err == -202) 
    {
      // No permission to the chrome & components directories
      alert('Error: Could not install ' + this.extFullName + ' ' + this.extVersion + '\n' + 
        EInstall_202_1 + '\n   ' + this.chromeDir + '\n' + EInstall_202_2 + '\n   ' + this.componentsDir);
    } else 
    if (err == -210) 
    {
      // Installation canceled by user
      if (!this.silentInstall)
        alert(EInstall_210_1);
    } else 
    {
      // Other unknown error
      alert(EInstallUnknown_1 + ' ' + err + '\n' + EInstallUnknown_2 + '\n' + EInstallUnknown_3);
    }
  }
};

XpiInstaller.install();